require 'sketchup.rb'

class Facewarp

  def initialize
    @ip0   = nil
    @ip1   = nil
    @ip2   = nil
    @ip3   = nil
    @xdown = 0
    @ydown = 0
  end

  def activate
    @ip0   = Sketchup::InputPoint.new
    @ip1   = Sketchup::InputPoint.new
    @ip2   = Sketchup::InputPoint.new
    @ip3   = Sketchup::InputPoint.new
    @step  = 0
    @xsubs = 2
    @ysubs = 2
  end

  def deactivate(view)
    view.invalidate
  end

  def onMouseMove(flags, x, y, view)

    if @step == 0
      @ip0.pick view, x, y
      view.invalidate
    else
      if @step == 1
        @ip1.pick view, x, y
        view.invalidate
      else
        if @step == 2
          @ip2.pick view, x, y
          view.invalidate
        else
          if @step == 3
            @ip3.pick view, x, y
            view.invalidate
          end
        end
      end
    end

  end
  
  def onCancel(reason, view)
    if reason == 0
      if @step == 1
        @ip1.clear
        @step = 0
      else
        if @step == 2
          @ip2.clear
          @step = 1
        else
          if @step == 3
            @ip3.clear
            @step = 2
          else
            if @step == 4
              @ip3.clear
              @step = 3
            end
          end
        end
      end
    end
  end

  def onLButtonDown(flags, x, y, view)

    if @step == 0
      @ip0.pick view, x, y
      @step = 1
    else
      if @step == 1
        @ip1.pick view, x, y
        @step = 2
      else
        if @step == 2
          @ip2.pick view, x, y
          @step = 3
        else
          if @step == 3
            @ip3.pick view, x, y
            Sketchup.set_status_text(@xsubs.to_s+" X "+@ysubs.to_s, SB_VCB_VALUE)
            @step = 4
          end
        end
      end
    end

  end

  def onKeyDown(key, repeat, flags, view)
    if key == VK_UP
      @ysubs = @ysubs + 1
    else
      if key == VK_DOWN
        @ysubs = @ysubs - 1 if @ysubs >= 2
      else
        if key == VK_LEFT
          @xsubs = @xsubs - 1 if @xsubs >= 2
        else
          if key == VK_RIGHT
            @xsubs = @xsubs + 1
          end
        end
      end
    end
    Sketchup.set_status_text(@xsubs.to_s+" X "+@ysubs.to_s, SB_VCB_VALUE)
    view.invalidate
  end

  def onReturn(view)
    if @step == 4
      model     = Sketchup.active_model
      operation = model.start_operation("Make a curved face")
      entities  = model.active_entities
      group     = entities.add_group
      groupents = group.entities
      
      boundery  = groupents.add_edges(@ip0.position,@ip1.position,@ip2.position,@ip3.position,@ip0.position)

      (1..@xsubs-1).each { |i|
        distance = (1.0/@xsubs.to_f)*i
        pt1  = Geom.linear_combination distance, @ip0.position, 1.0-distance, @ip1.position
        pt2  = Geom.linear_combination distance, @ip3.position, 1.0-distance, @ip2.position
        line = groupents.add_line(pt1, pt2)
      }

      (1..@ysubs-1).each { |i|
        distance = (1.0/@ysubs.to_f)*i
        pt1  = Geom.linear_combination distance, @ip3.position, 1.0-distance, @ip0.position
        pt2  = Geom.linear_combination distance, @ip2.position, 1.0-distance, @ip1.position
        line = groupents.add_line(pt1, pt2)
      }
      
      (0..@xsubs-1).each { |i|
        (0..@ysubs-1).each { |j|
          distancex = (1.0/@xsubs.to_f)*i
          distancey = (1.0/@ysubs.to_f)*j
          pt1 = Geom.linear_combination distancex, @ip0.position, 1.0-distancex, @ip1.position
          pt2 = Geom.linear_combination distancex, @ip3.position, 1.0-distancex, @ip2.position
          pt3 = Geom.linear_combination distancey,  pt1,          1.0-distancey,  pt2
          
          distancex = (1.0/@xsubs.to_f)*(i+1)
          distancey = (1.0/@ysubs.to_f)*(j+1)
          pt4 = Geom.linear_combination distancex, @ip0.position, 1.0-distancex, @ip1.position
          pt5 = Geom.linear_combination distancex, @ip3.position, 1.0-distancex, @ip2.position
          pt6 = Geom.linear_combination distancey,  pt4,          1.0-distancey,  pt5
          
          line = groupents.add_line(pt3, pt6)
          face = line.find_faces
        }
      }
      model.commit_operation
      @ip0.clear
      @ip1.clear
      @ip2.clear
      @ip3.clear
      @step = 0
    end
  end

  def draw(view)
    if( @ip0.valid? )
      @ip0.draw(view) if( @ip0.display? )
      if( @ip1.valid? )
        @ip1.draw(view) if( @ip1.display? )
        self.draw_geometry(@ip0.position, @ip1.position, view)

        if( @ip2.valid? )
          @ip2.draw(view) if( @ip2.display? )
          self.draw_geometry(@ip1.position, @ip2.position, view)

          if not ( @ip3.valid? )
            self.draw_geometry(@ip2.position, @ip0.position, view)
          end

          if( @ip3.valid? )
            @ip3.draw(view) if( @ip3.display? )
            self.draw_geometry(@ip2.position, @ip3.position, view)

            self.draw_geometry(@ip3.position, @ip0.position, view)

            (1..@xsubs-1).each { |i|
              view.drawing_color = "black"
              distance = (1.0/@xsubs.to_f)*i
              pt1 = Geom.linear_combination distance, @ip0.position, 1.0-distance, @ip1.position
              pt2 = Geom.linear_combination distance, @ip3.position, 1.0-distance, @ip2.position
              self.draw_geometry(pt1, pt2, view)
            }

            (1..@ysubs-1).each { |i|
              view.drawing_color = "black"
              distance = (1.0/@ysubs.to_f)*i
              pt1 = Geom.linear_combination distance, @ip3.position, 1.0-distance, @ip0.position
              pt2 = Geom.linear_combination distance, @ip2.position, 1.0-distance, @ip1.position
              self.draw_geometry(pt1, pt2, view)
            }

          end
        end
      end
    end
  end

  def draw_geometry(pt1, pt2, view)
    view.draw_line(pt1, pt2)
  end

end

menu = UI.menu("Plugins").add_item("Facewarp") { Sketchup.active_model.select_tool Facewarp.new }